#!/usr/bin/env python2
# -*- coding: utf-8 -*-

import time, sys, os
import grpc
from exchange_pb2_grpc import ApiStub
import exchange_pb2 as pb
import sqlalchemy as db
from optparse import OptionParser

def connect(url):
    ''' Подключиться к БД кассового сервера '''
    engine = db.create_engine('mysql://%s/documentsAll' % url)
    connection = engine.connect()
    securityweight = db.Table('securityweight', db.MetaData(), autoload=True, autoload_with=engine)
    return connection, securityweight

def barcodesQuery(securityweight):
    ''' Подготовить запрос списка товаров для которых есть взвешивания '''
    return db.select([securityweight.columns.barcode]).group_by(securityweight.columns.barcode)

def weightsQuery(securityweight, barcode):
    ''' Получить взвешивания для товара '''
    return db.select([securityweight.columns.weighing_date, securityweight.columns.source, securityweight.columns.weight]). \
        where(securityweight.columns.barcode == barcode).order_by(db.desc(securityweight.columns.weighing_date)).limit(100)

def sendProduct(stub, barcode, weights):
    ''' Загрузить товар на SCO '''
    request = pb.AddWeightsRequest()
    product = request.products.add()
    product.barcode = barcode
    product.datetime = int(round(time.time() * 1000))
    for weight in weights:
        w = product.weights.add()
        w.datetime = int(round(time.mktime(weight[0].utctimetuple()) * 1000))
        w.source = int(weight[1])
        w.weight = int(weight[2] * 1000)
    reply = stub.addWeights(request)

class Progressbar:
    ''' Отображает прогресс в консоли '''
    terminalWidth = int(os.popen('stty size', 'r').read().split()[1])
    def __init__(self, total, prefix):
        self.total = total
        self.prefix = prefix
        self.length = self.terminalWidth - len(prefix) - 10

    def update(self, iteration):
        percent = ("{:.1f}").format(100 * (iteration / float(self.total)))
        filledLength = int(self.length * iteration // self.total)
        bar = '#' * filledLength + '-' * (self.length - filledLength)
        sys.stdout.write('\r%s [%s] %s%%' % (self.prefix, bar, percent))
        sys.stdout.flush()
        if iteration == self.total:
            print ''

if __name__ == '__main__':
    parser = OptionParser(description = u'Миграция данных контроля веса с кассового сервера Artix на кассу самообслуживания SST')
    parser.add_option("-c", "--cash", action="store", type="string", dest="cash", default="localhost:3201", help=u"Адрес кассы на которую будут выгружены данные. Формат адреса: 'address:port' [По умолчанию: %default]")
    parser.add_option("-s", "--server", action="store", type="string", dest="server", help=u"Адрес кассового сервера с которого будут выгружены данные. Формат адреса: 'username:password@address:port'")
    (options, args) = parser.parse_args()
    if not options.server:
        parser.error(u'Не указан адрес кассового сервера')

    channel = grpc.insecure_channel(options.cash)
    stub = ApiStub(channel)
    connection, securityweight = connect(options.server)
    barcodes = connection.execute(barcodesQuery(securityweight)).fetchall()
    count = len(barcodes)
    progress = Progressbar(count, u"Выгрузка взвешиваний:")
    for i in range(count):
        progress.update(i + 1)
        weights = connection.execute(weightsQuery(securityweight, barcodes[i][0])).fetchall()
        sendProduct(stub, barcodes[i][0], weights)
